@@ -1,5 +1,4 @@
 from PyQt6.QtWidgets import QMessageBox
-import minecraft_launcher_lib
 from typing import Dict, List
 import subprocess
 import platform
@@ -7,7 +6,6 @@ import requests
 import shutil
 import socket
 import json
-import copy
 import os
 
 
@@ -49,7 +47,6 @@ def hasInternetConnection() -> bool:
         return True
     except OSError:
         return False
-    return False
 
 
 def downloadFile(url: str, path: str):
@@ -57,7 +54,7 @@ def downloadFile(url: str, path: str):
         return
     try:
         os.makedirs(os.path.dirname(path))
-    except:
+    except Exception:
         pass
     r = requests.get(url, stream=True)
     with open(path, 'wb') as f:
@@ -74,17 +71,21 @@ def getAccountDict(information_dict: Dict) -> Dict:
     }
 
 
+def _addJavaRuntimeDir(path: str, runtimeList: List[str]):
+    if not os.path.isdir(path):
+        return
+    for i in os.listdir(path):
+        if not os.path.islink(os.path.join(path, i)):
+            runtimeList.append(os.path.join(path, i, "bin", "java"))
+
+
 def findJavaRuntimes() -> List[str]:
     runtimeList = []
-    if os.path.isdir("/usr/lib/jvm"):
-        for i in os.listdir("/usr/lib/jvm"):
-            if not os.path.islink(os.path.join("/usr/lib/jvm", i)):
-                runtimeList.append(os.path.join("/usr/lib/jvm", i, "bin", "java"))
-    if os.path.isdir("/usr/lib/sdk"):
-        for i in os.listdir("/usr/lib/sdk"):
-            runtimeList.append(os.path.join("/usr/lib/sdk", i, "bin", "java"))
+    _addJavaRuntimeDir("/usr/lib/jvm", runtimeList)
+    _addJavaRuntimeDir("/usr/lib/sdk", runtimeList)
+    _addJavaRuntimeDir("/app/jvm", runtimeList)
     return runtimeList
 
 
 def isFlatpak() -> bool:
-    return os.path.isdir("/app")
+    return os.path.isfile("/.flatpak-info")
