@@ -1,10 +1,7 @@
 from PyQt6.QtWebEngineWidgets import QWebEngineView
-from PyQt6.QtWebEngineCore import QWebEngineProfile
-from PyQt6.QtWidgets import QApplication, QMessageBox
-from PyQt6.QtCore import QUrl, QLocale
+from PyQt6.QtCore import QUrl
 import minecraft_launcher_lib
 import json
-import sys
 import os
 
 
