@@ -1,18 +1,17 @@
-from PyQt6.QtWidgets import QWidget, QTableWidget, QTableWidgetItem, QVBoxLayout, QHBoxLayout, QGridLayout, QPlainTextEdit, QTabWidget, QAbstractItemView, QHeaderView, QPushButton, QComboBox, QProgressBar, QLabel, QCheckBox, QFileDialog, QMenu, QLineEdit, QInputDialog, QSizePolicy
-from PyQt6.QtCore import QUrl, QLocale, Qt, QDir, QProcess
+from PyQt6.QtWidgets import QWidget, QTableWidget, QTableWidgetItem, QVBoxLayout, QHBoxLayout, QGridLayout, QPlainTextEdit, QTabWidget, QAbstractItemView, QHeaderView, QPushButton, QComboBox, QProgressBar, QLabel, QCheckBox, QMenu, QLineEdit, QSizePolicy
+from PyQt6.QtCore import QUrl, QLocale, Qt, QProcess
 from PyQt6.QtWebEngineWidgets import QWebEngineView
 from PyQt6.QtWebEngineCore import QWebEngineProfile
-from PyQt6.QtGui import QPixmap, QCursor, QAction
+from PyQt6.QtGui import QCursor, QAction
 from jdMinecraftLauncher.gui.ProfileWindow import ProfileWindow
 from jdMinecraftLauncher.Profile import Profile
-from jdMinecraftLauncher.Functions import openFile, saveProfiles, showMessageBox, downloadFile
+from jdMinecraftLauncher.Functions import openFile, saveProfiles, showMessageBox
 from jdMinecraftLauncher.InstallThread import InstallThread
 from jdMinecraftLauncher.RunMinecraft import runMinecraft
 import minecraft_launcher_lib
 import webbrowser
 import tempfile
 import random
-import urllib
 import shutil
 import json
 import sys
@@ -206,6 +205,105 @@ class SwitchAccountButton(QPushButton):
             # self.env.loginWindow.setName(account.get("mail",""))
             self.env.loginWindow.show()
 
+class ForgeTab(QTableWidget):
+    def __init__(self, env, mainWindow):
+        self.env = env
+        self.mainWindow = mainWindow
+        super().__init__(0,2)
+
+        self.horizontalHeader().setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
+        self.horizontalHeader().setSectionResizeMode(1, QHeaderView.ResizeMode.Stretch)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
+        self.horizontalHeader().hide()
+        self.verticalHeader().hide()
+
+        count = 0
+        minecraft_version_check = {}
+        for i in minecraft_launcher_lib.forge.list_forge_versions():
+            minecraft_version, _ = i.split("-", 1)
+
+            if minecraft_version in minecraft_version_check or not minecraft_launcher_lib.forge.supports_automatic_install(i):
+                continue
+
+            minecraft_version_check[minecraft_version] = True
+
+            versionItem = QTableWidgetItem(i)
+            versionItem.setFlags(versionItem.flags() ^ Qt.ItemFlag.ItemIsEditable)
+
+            installButton = QPushButton(env.translate("mainwindow.button.install"))
+
+            installButton.clicked.connect(self.installButtonClicked)
+
+            self.insertRow(count)
+            self.setItem(count, 0, versionItem)
+            self.setCellWidget(count, 1, installButton)
+
+            count += 1
+
+    def installForgeVersion(self, forgeVersion: str):
+        self.mainWindow.installThread.setupForgeInstallation(forgeVersion)
+        self.mainWindow.installThread.start()
+
+        self.mainWindow.setInstallButtonsEnabled(False)
+
+    def installButtonClicked(self):
+        for i in range(self.rowCount()):
+            if self.cellWidget(i, 1) == self.sender():
+                self.installForgeVersion(self.item(i, 0).text())
+                return
+
+    def setButtonsEnabled(self, enabled: bool):
+         for i in range(self.rowCount()):
+             self.cellWidget(i, 1).setEnabled(enabled)
+
+
+class FabricTab(QTableWidget):
+    def __init__(self, env, mainWindow):
+        self.env = env
+        self.mainWindow = mainWindow
+        super().__init__(0,2)
+
+        self.horizontalHeader().setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
+        self.horizontalHeader().setSectionResizeMode(1, QHeaderView.ResizeMode.Stretch)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
+        self.horizontalHeader().hide()
+        self.verticalHeader().hide()
+
+        count = 0
+        for i in minecraft_launcher_lib.fabric.get_all_minecraft_versions():
+            if not i["stable"]:
+                continue
+
+            versionItem = QTableWidgetItem(i["version"])
+            versionItem.setFlags(versionItem.flags() ^ Qt.ItemFlag.ItemIsEditable)
+
+            installButton = QPushButton(env.translate("mainwindow.button.install"))
+
+            installButton.clicked.connect(self.installButtonClicked)
+
+            self.insertRow(count)
+            self.setItem(count, 0, versionItem)
+            self.setCellWidget(count, 1, installButton)
+
+            count += 1
+
+    def installFabricVersion(self, fabricVersion: str):
+        self.mainWindow.installThread.setupFabricInstallation(fabricVersion)
+        self.mainWindow.installThread.start()
+
+        self.mainWindow.setInstallButtonsEnabled(False)
+
+    def installButtonClicked(self):
+        for i in range(self.rowCount()):
+            if self.cellWidget(i, 1) == self.sender():
+                self.installFabricVersion(self.item(i, 0).text())
+                return
+
+    def setButtonsEnabled(self, enabled: bool):
+         for i in range(self.rowCount()):
+             self.cellWidget(i, 1).setEnabled(enabled)
+
+
 class AccountTab(QTableWidget):
     def __init__(self,env):
         self.env = env
@@ -225,7 +323,7 @@ class AccountTab(QTableWidget):
             nameItem.setFlags(nameItem.flags() ^ Qt.ItemFlag.ItemIsEditable)
             button = SwitchAccountButton(self.env.translate("accounttab.button.switch"),self.env,count)
             self.insertRow(count)
-            self.setItem(count, 0, nameItem) 
+            self.setItem(count, 0, nameItem)
             self.setCellWidget(count,1,button)
             # self.setItem(count, 1, versionItem) 
             count += 1
@@ -299,7 +397,7 @@ class GameOutputTab(QPlainTextEdit):
         if self.natives_path != "":
             try:
                 shutil.rmtree(self.natives_path)
-            except:
+            except Exception:
                 pass
 
     def executeCommand(self,profile, command, natives_path):
@@ -332,6 +430,11 @@ class Tabs(QTabWidget):
         self.addTab(self.versionTab,env.translate("mainwindow.tab.versionEditor"))
         self.options = OptionsTab(env)
         self.addTab(self.options,env.translate("mainwindow.tab.options"))
+        if not env.offlineMode:
+            self.forgeTab = ForgeTab(env, parrent)
+            self.addTab(self.forgeTab, "Forge")
+            self.fabricTab = FabricTab(env, parrent)
+            self.addTab(self.fabricTab, "Fabric")
         self.accountTab = AccountTab(env)
         self.addTab(self.accountTab,"Account")
         about = AboutTab(env)
@@ -462,9 +565,14 @@ class MainWindow(QWidget):
         o.executeCommand(profile,args,natives_path)
 
     def installFinish(self):
-        self.env.updateInstalledVersions()
-        self.tabWidget.versionTab.updateVersions()
-        self.startMinecraft(self.env.current_running_profile)
+        if self.installThread.shouldStartMinecraft():
+            self.env.updateInstalledVersions()
+            self.tabWidget.versionTab.updateVersions()
+            self.startMinecraft(self.env.current_running_profile)
+        else:
+            self.env.loadVersions()
+            self.profileWindow.updateVersionsList
+            self.setInstallButtonsEnabled(True)
 
     def installVersion(self,profile):
         self.env.current_running_profile = profile
@@ -480,6 +588,11 @@ class MainWindow(QWidget):
         else:
             self.env.translate("mainwindow.button.play")
 
+    def setInstallButtonsEnabled(self, enabled: bool):
+        self.playButton.setEnabled(enabled)
+        self.tabWidget.forgeTab.setButtonsEnabled(enabled)
+        self.tabWidget.fabricTab.setButtonsEnabled(enabled)
+
     def closeEvent(self,event):
         options = self.tabWidget.options
         self.env.settings.language = options.languageComboBox.currentData()
