@@ -1,12 +1,10 @@
 from jdMinecraftLauncher.Functions import hasInternetConnection, showMessageBox, getAccountDict
-from jdMinecraftLauncher.MicrosoftSecrets import MicrosoftSecrets
 from jdMinecraftLauncher.gui.LoginWindow import LoginWindow
 from jdMinecraftLauncher.gui.MainWindow import MainWindow
 from jdMinecraftLauncher.Enviroment import Enviroment
 from PyQt6.QtWidgets import QApplication
 from PyQt6.QtGui import QIcon
 import minecraft_launcher_lib
-import json
 import copy
 import sys
 import os
@@ -14,7 +12,11 @@ import os
 def main():
     app = QApplication(sys.argv)
     env = Enviroment()
+
+    app.setApplicationName("jdMinecraftLauncher")
+    app.setDesktopFileName("com.gitlab.JakobDev.jdMinecraftLauncher")
     app.setWindowIcon(QIcon(os.path.join(env.currentDir , "Icon.svg")))
+
     env.mainWindow = MainWindow(env)
     env.loginWindow = LoginWindow(env)
 
